Attribute VB_Name = "modAPI"
Option Explicit

'-- Declaracio de funcions API
Private Declare Function GetPrivateProfileString Lib "Kernel32" Alias "GetPrivateProfileStringA" (ByVal lpApplicationName As String, ByVal lpKeyName As Any, ByVal lpDefault As String, ByVal lpReturnedString As String, ByVal nSize As Long, ByVal lpFileName As String) As Long
Private Declare Function WritePrivateProfileString Lib "Kernel32" Alias "WritePrivateProfileStringA" (ByVal lpApplicationName As String, ByVal lpKeyName As Any, ByVal lpString As Any, ByVal lpFileName As String) As Long
Private Declare Function CopyFile Lib "Kernel32" Alias "CopyFileA" (ByVal lpExistingFileName As String, ByVal lpNewFileName As String, ByVal bFailIfExists As Long) As Long
Private Declare Function ShellExecute Lib "shell32.dll" Alias "ShellExecuteA" (ByVal hwnd As Long, ByVal lpszOp As String, ByVal lpszFile As String, ByVal lpszParams As String, ByVal lpszDir As String, ByVal FsShowCmd As Long) As Long
Private Declare Function GetDesktopWindow Lib "user32" () As Long
Private Declare Function GetCurrentProcess Lib "Kernel32" () As Long
Private Declare Function OpenProcessToken Lib "advapi32" (ByVal ProcessHandle As Long, ByVal DesiredAccess As Long, TokenHandle As Long) As Long
Private Declare Function LookupPrivilegeValue Lib "advapi32" Alias "LookupPrivilegeValueA" (ByVal lpSystemName As String, ByVal lpName As String, lpLuid As LUID) As Long
Private Declare Function AdjustTokenPrivileges Lib "advapi32" (ByVal TokenHandle As Long, ByVal DisableAllPrivileges As Long, NewState As TOKEN_PRIVILEGES, ByVal BufferLength As Long, PreviousState As TOKEN_PRIVILEGES, ReturnLength As Long) As Long
Private Declare Function ExitWindowsEx Lib "user32" (ByVal dwOptions As Long, ByVal dwReserved As Long) As Long
Private Declare Function FindWindow Lib "user32" Alias "FindWindowA" (ByVal lpClassName As String, ByVal lpWindowName As String) As Long
Private Declare Sub Sleep Lib "Kernel32" (ByVal dwMilliseconds As Long)
'-- Declaracio de constants
Private Const SW_SHOWNORMAL = 1
Public Const FILE_ATTRIBUTE_NORMAL = &H80
Private Const TOKEN_ADJUST_PRIVILEGES = &H20
Private Const TOKEN_QUERY = &H8
Private Const SE_PRIVILEGE_ENABLED = &H2
Private Const EWX_SHUTDOWN As Long = 1
Private Const EWX_FORCE As Long = 4
Private Const EWX_REBOOT = 2
'-- Declaracio d'estructures
Private Type LUID
    UsedPart As Long
    IgnoredForNowHigh32BitPart As Long
End Type
Private Type TOKEN_PRIVILEGES
  PrivilegeCount As Long
  TheLuid As LUID
  Attributes As Long
End Type

'--------------------------------------------------------------
'-- ObrirElPrograma()
'--------------------------------------------------------------
' Mitjanant la funcio ShellExecute de les API del Sistema,
' puc obrir un programa, obrir un nou correu electronic, etc.
'--------------------------------------------------------------
Public Function ObrirElPrograma(strNomDoc As String) As Long
Dim lngScr_hDC As Long
   
   On Error GoTo TractaError
   lngScr_hDC = GetDesktopWindow()
   ObrirElPrograma = ShellExecute(lngScr_hDC, "Open", strNomDoc, "", "C:\", SW_SHOWNORMAL)

Exit Function
TractaError:
   ObrirElPrograma = 0
End Function

'--------------------------------------------------------------
'-- INI_Get()
'--------------------------------------------------------------
' Permet llegir un fitxer de configuracio
'--------------------------------------------------------------
Public Function INI_Get(ByVal strParametre As String, ByVal varDefecte As Variant, Optional Grup As Variant, Optional Fitxer As Variant, Optional BooleaTrue As Variant, Optional BooleaFalse As Variant) As Variant
Static strGrup As String
Static strFitxer As String
Dim intNumCar As Integer
Dim strBuffer As String

  '-- miro els parametres per defecte de booleans
  If IsMissing(BooleaTrue) Then BooleaTrue = "ON,SI"
  If IsMissing(BooleaFalse) Then BooleaFalse = "OFF,NO"
  '-- miro els parametres per defecte de grup i fitxer
  If Not IsMissing(Grup) Then strGrup = Grup
  If Not IsMissing(Fitxer) Then strFitxer = Fitxer
  
  '----inicialitzem el buffer
  strBuffer = String(1200, Chr$(0))
  
  '---- obrim INI i capturem valor
  intNumCar = GetPrivateProfileString(strGrup, strParametre, "", strBuffer, Len(strBuffer), strFitxer)
  
  '-- mirem si no trobat, retorna valor per defecte
  If intNumCar = 0 Then
    
    '-- retorno el valor per defecte
    INI_Get = varDefecte

  Else
    
    '-- Trec els caracters nuls de la cadena de caracters
    strBuffer = Mid$(strBuffer, 1, intNumCar)
      
    Select Case VarType(varDefecte)

      Case 2  'Integer
        '-- contrueixo una cadena de caracters on hi han les posibles
        '-- combinacions de valors en INI que indiquen Valor Boole.
        '-- buscar el que ha retornat dins la cadena, si hi es vol dir
        '-- que es un boole i mirar si es un valor de true o false
        If InStr(BooleaTrue & "," & BooleaFalse, UCase(Trim$(strBuffer))) Then
          '-- retorno True o False segons el que hgim llegit
          INI_Get = IIf(InStr(BooleaTrue, UCase(Trim$(strBuffer))), True, False)
        Else
          '-- retorno el valor enter
          INI_Get = Val(Trim$(strBuffer))
        End If

      Case 3  'Long
        INI_Get = CLng(strBuffer)
      Case 4  'Single
        INI_Get = CSng(strBuffer)
      Case 5  'Double
        INI_Get = CDbl(strBuffer)
      Case 6  'Currency
        INI_Get = CCur(strBuffer)
      Case 8  'String
        INI_Get = Trim$(strBuffer)
      Case 7  'Date
        INI_Get = CVDate(strBuffer)
      Case 11
        '-- retorno True o False segons el que hagim llegit
        INI_Get = InStr(UCase(BooleaTrue), UCase(Trim$(strBuffer))) <> 0
      Case Else
        INI_Get = 0
    End Select

  End If

End Function

'--------------------------------------------------------------
'-- INI_Put()
'--------------------------------------------------------------
' Permet escriure un fitxer de configuracio
'--------------------------------------------------------------
Public Sub INI_Put(Identificador As String, ByVal Cadena As Variant, Optional Grup As Variant, Optional Fitxer As Variant)
Static CopiaGrup As String
Static CopiaFitxer As String
Dim a As Integer

  If Not IsMissing(Grup) Then CopiaGrup = Grup
  If Not IsMissing(Fitxer) Then CopiaFitxer = Fitxer
  
  a = WritePrivateProfileString(CopiaGrup, Identificador, CStr(Cadena), CopiaFitxer)
  
End Sub

'------------------------------------------
'-- CopiarFitxer()
'------------------------------------------
' Permet copiar un fitxer origen a un fitxer desti
'------------------------------------------
Public Function CopiarFitxer(strOrigen As String, strDesti As String) As Boolean
Dim lngRet As Long
Dim lngErr As Long

   On Error GoTo TractaError
   '-- Copio el fitxer origen en el desti
   lngRet = CopyFile(strOrigen, strDesti, lngErr)
   CopiarFitxer = Not lngRet
   
Exit Function
TractaError:
   CopiarFitxer = False
   Exit Function
End Function

Public Function Base10BaseN(lngValor As Long, intBase As Integer) As String
Dim lngTmp As Long
Dim strTmp As String

  '-- Comprovem la precondicio d'entrada
  If lngValor >= 0 And intBase > 1 And intBase <= 37 Then
    '-- em deso el valor
    lngTmp = lngValor
    strTmp = ""
    '-- bucle per anar dividint
    Do
      '-- poso el carcater que correspon a la resta de la divisio
      strTmp = CaracterBase10BaseN(lngTmp Mod intBase) & strTmp
      '-- faig la divisio entera
      lngTmp = lngTmp \ intBase
    Loop Until lngTmp < intBase
    '-- poso el caracter que correspon al valor enter de la divisio
    If lngTmp > 0 Then strTmp = CaracterBase10BaseN(CInt(lngTmp)) & strTmp
    '-- retorno la cadena convertida
    Base10BaseN = strTmp
  End If

End Function

Public Function BaseNBase10(strValor As String, intBase As Integer) As Long
Dim Index As Integer
Dim strTmp As String
Dim lngTmp As Long
Dim intDigit As Integer
Dim strDigit As String * 1

  '-- Faig una copia interna del valor, sense espais
  strTmp = Trim$(strValor)
  '-- Comprovo la precondicio d'entrada
  If strTmp <> "" Then
    '-- Inicialitzo el valor
    lngTmp = 0
    '-- Bucle per calcular el valor de cada digit
    For Index = 0 To Len(strTmp) - 1
      '-- Miro el digit
      strDigit = Mid$(strTmp, Len(strTmp) - Index, 1)
      '-- Miro el valor del digit
      intDigit = CaracterBaseNBase10(strDigit)
      '-- Miro el que val aquest digit
      lngTmp = lngTmp + ((intBase ^ Index) * intDigit)
    Next Index
    '-- Retorno el valor
    BaseNBase10 = lngTmp
  End If

End Function

Public Function BaseNBaseN(strValor As String, intBaseOrigen As Integer, intBaseDesti As Integer) As String

  '-- Per fer la conversio utilitzo les funcions passant per base 10
  BaseNBaseN = Base10BaseN(BaseNBase10(strValor, intBaseOrigen), intBaseDesti)

End Function

'------------------------------------------'
' BitEstatGet()
'------------------------------------------'
' Retorna l'estat d'un dels bits d'un word '
'------------------------------------------'
Public Function BitEstatGet(Word As Double, Bit As Integer) As Integer
Dim bolEstat As Boolean

   If Word >= 0 Then
      'True si esta activat, False en cas contrari
      bolEstat = (Word And 2 ^ Bit) = 2 ^ Bit
      'Retorno 1 si esta activat, 0 en cas contrari
      BitEstatGet = IIf(bolEstat, 1, 0)
   Else
      BitEstatGet = 0
   End If
End Function

'--------------------------------------------------'
' BitEstatSet()
'--------------------------------------------------'
' Aquesta funcio serveix per activar o desactivar  '
' un bit d'un word.                                '
' Segons el valor boolea d'estat activara o        '
' desactivara el bit que li passem al parametre    '
' Bit del word que li passem a Word.         '
' Si volem activar un bit que esta activat,        '
' el bit retorna activat. Si volem desactivar      '
' un bit que esta desactivat el retorna desactivat '
' La funcio no conmutara l'estat del bit.          '
'--------------------------------------------------'
Public Function BitEstatSet(Word As Double, Bit As Integer, Estat As Integer) As Double

  '-- Miro si volem activar el bit
  If Estat = 1 Then
    '-- Miro si el bit esta Desactivat
    If (Word And 2 ^ Bit) <> 2 ^ Bit Then
      '-- L'activo
      Word = Word Xor 2 ^ Bit
    End If
  Else
    '-- Miro si el bit esta activat
    If (Word And 2 ^ Bit) = 2 ^ Bit Then
      '-- El desactivo
      Word = Word Xor 2 ^ Bit
    End If
  End If
  '-- Retorno el resultat
  BitEstatSet = Word
  
End Function

Private Function CaracterBase10BaseN(Valor As Integer) As String

  Select Case Valor
    Case 0 To 9
      CaracterBase10BaseN = CStr(Valor)
    Case 10 To 36
      CaracterBase10BaseN = Chr$(Valor + 87)
    Case Else
      CaracterBase10BaseN = ""
  End Select

End Function

Private Function CaracterBaseNBase10(strValor As String) As Integer
Dim strTmp As String * 1

  strTmp = LCase(strValor)
  Select Case Asc(strTmp)
    '-- Numeros
    Case 48 To 57
      CaracterBaseNBase10 = Asc(strTmp) - 48
    '-- Lletres
    Case 97 To 122
      CaracterBaseNBase10 = Asc(strTmp) - 87
    Case Else
      CaracterBaseNBase10 = -1
  End Select

End Function

Public Sub APAGAR_W2K(reiniciar)
Dim hdlProcessHandle As Long
Dim hdlTokenHandle As Long
Dim tmpLuid As LUID
Dim tkp As TOKEN_PRIVILEGES
Dim tkpNewButIgnored As TOKEN_PRIVILEGES
Dim lBufferNeeded As Long
Dim lRetVal As Long

   On Error GoTo suberror

   hdlProcessHandle = GetCurrentProcess()
   OpenProcessToken hdlProcessHandle, (TOKEN_ADJUST_PRIVILEGES Or TOKEN_QUERY), hdlTokenHandle

   ' Get the LUID for shutdown privilege.
   LookupPrivilegeValue "", "SeShutdownPrivilege", tmpLuid

   tkp.PrivilegeCount = 1    ' One privilege to set
   tkp.TheLuid = tmpLuid
   tkp.Attributes = SE_PRIVILEGE_ENABLED

   ' Enable the shutdown privilege in the access token of this process.
   AdjustTokenPrivileges hdlTokenHandle, False, _
   tkp, Len(tkpNewButIgnored), tkpNewButIgnored, lBufferNeeded
    
   If reiniciar = 1 Then
      lRetVal = ExitWindowsEx(EWX_REBOOT Or EWX_FORCE, &HFFFF)
   Else
      lRetVal = ExitWindowsEx(EWX_SHUTDOWN Or EWX_FORCE, &HFFFF)
   End If

Exit Sub
suberror:
   End
End Sub

Public Function MiraSiAppEstaFuncionant(strApp As String) As Boolean
Dim lhwnd As Long

   lhwnd = FindWindow(vbNullString, strApp)
   If lhwnd > 0 Then
      MiraSiAppEstaFuncionant = True
   End If

End Function

Public Sub Delay(lngMilisegons As Long)
   Sleep lngMilisegons
End Sub

